﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NationalInstruments.ModularInstruments.Interop;

namespace Example_1
{
    class Program
    {
        /// <summary>
        /// Using a RFSA VISA session and a status code the status code is checked to see if a warning has occured.
        /// </summary>
        /// <param name="Rfsa">RFSA instance</param>
        /// <param name="Status">Status code</param>
        static private void ErrorCheckRFSA(niRFSA Rfsa, int Status)
        {
            StringBuilder sb = new StringBuilder(256);

            if (Status > 0) // Warning
            {
                // Retrieve the error message based on the status code.
                Rfsa.ErrorMessage(Status, sb);

                Console.WriteLine("Warning : " + sb.ToString());
            }          
        }

        /// <summary>
        ///  Using a RFSG VISA session and a status code the status code is checked to see if a warning has occured.
        /// </summary>
        /// <param name="Rfsg">RFSG instance</param>
        /// <param name="Status">Status code</param>
        static private void ErrorCheckRFSG(niRFSG Rfsg, int Status)
        {
            StringBuilder sb = new StringBuilder(256);

            if (Status > 0) // Warning
            {
                // Retrieve the error message based on the status code.
                Rfsg.ErrorMessage(Status, sb);

                Console.WriteLine("Warning : " + sb.ToString());
            }           
        }

        /// <summary>
        /// Boots the RFSA within the VST.
        /// </summary>
        /// <param name="Rfsa">Instance of the Rfsa object.</param>
        static private void BootRFSA(out niRFSA Rfsa)
        {
            int status;
            double referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

            // Initiate VSA session		
            Rfsa = new niRFSA("RIO0",true,false);
           
            // Configure 10MHz reference
            status = Rfsa.ConfigureRefClock("OnboardClock", referenceClockRate); // "OnboardClock" or "RefIn" or "PXI_CLK" or "None"
            ErrorCheckRFSA(Rfsa,status);

            // Commit the settings to make clock settings active
            status = Rfsa.Commit();
            ErrorCheckRFSA(Rfsa,status);

            Console.WriteLine("VST - RFSA Booted and Configured for on-board 10 MHz reference clock. : ");            
        }

        /// <summary>
        /// Boots the RFSG within the VST.
        /// </summary>
        /// <param name="Rfsa">Instance of the Rfsg object.</param>
        static private void BootRFSG(out niRFSG Rfsg)
        {
            int status;
            double referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

            // Initiate VSA session		
            Rfsg = new niRFSG("RIO0", true, false);

            // Configure 10MHz reference
            status = Rfsg.ConfigureRefClock("OnboardClock", referenceClockRate); // "OnboardClock" or "RefIn" or "PXI_CLK" or "None"
            ErrorCheckRFSG(Rfsg, status);

            // Commit the settings to make clock settings active
            status = Rfsg.Commit();
            ErrorCheckRFSG(Rfsg, status);

            Console.WriteLine("VST - RFSG Booted and Configured for on-board 10 MHz reference clock. : ");
        }

        
        /// <summary>
        /// Closes the RFSA session.
        /// </summary>
        /// <param name="Rfsa">RFSA instance</param>
        static private void CloseRFSA(niRFSA Rfsa)
        {
            if (Rfsa != null)
            {
                // Close session
                Rfsa.Close();
            }
        }

        /// <summary>
        /// Closes the RFSG session.
        /// </summary>
        /// <param name="Rfsg">RFSG instance.</param>
        static private void CloseRFSG(niRFSG Rfsg)
        {
            if (Rfsg != null)
            {
                // Close session
                Rfsg.close();
            }
        }
        
        /// <summary>
        /// Console App entry point.
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            niRFSG Rfsg = null;
            niRFSA Rfsa = null;

            try
            {
                // Set the window size a little larger to make things easier to read.
                Console.SetWindowSize(150, 50);
                Console.WriteLine("VST Example 1 - Booting, Configuring and Closing");
                Console.WriteLine("------------------------------------------------");
                Console.WriteLine();
                Console.WriteLine("Booting and configuring RFSA.");
                BootRFSA(out Rfsa);
                Console.WriteLine("Booting and configuring RFSG.");
                BootRFSG(out Rfsg);
            }
            catch (Exception ex)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(ex.Message);
                Console.ForegroundColor = ConsoleColor.Gray;
            }
            finally
            {
                Console.WriteLine("Closing RFSA.");
                CloseRFSA(Rfsa);
                Console.WriteLine("Closing RFSG.");
                CloseRFSG(Rfsg);
                Console.WriteLine();
                Console.WriteLine();
                Console.WriteLine("Press Any Key To Exit!");
                Console.ReadLine();
            }
        }
    }
}
