﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using NationalInstruments.ModularInstruments.Interop;

namespace Example_1
{
    class Program
    {
        /// <summary>
        /// Using a RFSA VISA session and a status code the status code is checked to see if a warning has occured.
        /// </summary>
        /// <param name="Rfsa">RFSA instance</param>
        /// <param name="Status">Status code</param>
        static private void ErrorCheckRFSA(niRFSA Rfsa, int Status)
        {
            StringBuilder sb = new StringBuilder(256);

            if (Status > 0) // Warning
            {
                // Retrieve the error message based on the status code.
                Rfsa.ErrorMessage(Status, sb);

                Console.WriteLine("Warning : " + sb.ToString());
            }          
        }

        /// <summary>
        ///  Using a RFSG VISA session and a status code the status code is checked to see if a warning has occured.
        /// </summary>
        /// <param name="Rfsg">RFSG instance</param>
        /// <param name="Status">Status code</param>
        static private void ErrorCheckRFSG(niRFSG Rfsg, int Status)
        {
            StringBuilder sb = new StringBuilder(256);

            if (Status > 0) // Warning
            {
                // Retrieve the error message based on the status code.
                Rfsg.ErrorMessage(Status, sb);

                Console.WriteLine("Warning : " + sb.ToString());
            }           
        }

        /// <summary>
        /// Boots the RFSA within the VST.
        /// </summary>
        /// <param name="Rfsa">Instance of the Rfsa object.</param>
        static private void BootRFSA(out niRFSA Rfsa)
        {
            int status;
            double referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

            // Initiate VSA session		
            Rfsa = new niRFSA("RIO0",true,false);
           
            // Configure 10MHz reference
            status = Rfsa.ConfigureRefClock("OnboardClock", referenceClockRate); // "OnboardClock" or "RefIn" or "PXI_CLK" or "None"
            ErrorCheckRFSA(Rfsa,status);

            // Commit the settings to make clock settings active
            status = Rfsa.Commit();
            ErrorCheckRFSA(Rfsa,status);

            Console.WriteLine("VST - RFSA Booted and Configured for on-board 10 MHz reference clock. : ");            
        }

        /// <summary>
        /// Boots the RFSG within the VST.
        /// </summary>
        /// <param name="Rfsa">Instance of the Rfsg object.</param>
        static private void BootRFSG(out niRFSG Rfsg)
        {
            int status;
            double referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

            // Initiate VSA session		
            Rfsg = new niRFSG("RIO0", true, false);

            // Configure 10MHz reference
            status = Rfsg.ConfigureRefClock("OnboardClock", referenceClockRate); // "OnboardClock" or "RefIn" or "PXI_CLK" or "None"
            ErrorCheckRFSG(Rfsg, status);

            // Commit the settings to make clock settings active
            status = Rfsg.Commit();
            ErrorCheckRFSG(Rfsg, status);

            Console.WriteLine("VST - RFSG Booted and Configured for on-board 10 MHz reference clock. : ");
        }

        
        /// <summary>
        /// Closes the RFSA session.
        /// </summary>
        /// <param name="Rfsa">RFSA instance</param>
        static private void CloseRFSA(niRFSA Rfsa)
        {
            if (Rfsa != null)
            {
                // Close session
                Rfsa.Close();
            }
        }

        /// <summary>
        /// Closes the RFSG session.
        /// </summary>
        /// <param name="Rfsg">RFSG instance.</param>
        static private void CloseRFSG(niRFSG Rfsg)
        {
            if (Rfsg != null)
            {
                // Close session
                Rfsg.close();
            }
        }

        //****************************************************************************** 
        // Function          : Capture
        // Globals Used      : None
        // Globals Modified  : None
        // Description       : Captures IQ for a specified dwell time at a specified frequency and input level at a specified sample rate.
        //******************************************************************************

        /// <summary>
        /// Captures IQ for a specified dwell time at a specified frequency and input level at a specified sample rate.
        /// </summary>
        /// <param name="Rfsa">VISA instance of RFSA</param>
        /// <param name="CarrierFrequency"></param>
        /// <param name="ReferenceLevel"></param>
        /// <param name="NumSamples"></param>
        /// <param name="SampleRate"></param>
        /// <param name="iBuffer"></param>
        /// <param name="qBuffer"></param>
        static private void Capture(niRFSA Rfsa, double CarrierFrequency, double ReferenceLevel, int NumSamples, double SampleRate, double[] iBuffer, double[] qBuffer)
        {
            int status;
            niRFSA_wfmInfo captureInfo;
            double timeout = 10.0;            

            //
            // Configure VST for capture
            //
            status = Rfsa.ConfigureAcquisitionType(niRFSAConstants.Iq); // IQ capture mode
            ErrorCheckRFSA(Rfsa, status);

            // Set Frequency in Hz
            status = Rfsa.ConfigureIQCarrierFrequency("", CarrierFrequency);
            ErrorCheckRFSA(Rfsa, status);

            // Set Level in dBm
            status = Rfsa.ConfigureReferenceLevel("", ReferenceLevel); 
            ErrorCheckRFSA(Rfsa, status);

            // Set sample rate in Hz.
            status = Rfsa.ConfigureIQRate("", SampleRate);
            ErrorCheckRFSA(Rfsa, status);

            // Number of samples
            status = Rfsa.ConfigureNumberOfSamples("", true, NumSamples); 
            ErrorCheckRFSA(Rfsa, status);

            // Number of records
            status = Rfsa.ConfigureNumberOfRecords("", true, 1); 
            ErrorCheckRFSA(Rfsa, status);

            // No Start Triggers
            status = Rfsa.DisableStartTrigger();
            ErrorCheckRFSA(Rfsa, status);

            // Start capture
            status = Rfsa.Initiate(); 
            ErrorCheckRFSA(Rfsa, status);           

            if (status == 0)
            {
                // Create an array of Complex doubles to store the IQ data returned from the RFSA.
                niComplexNumber[] capturedIQ = new niComplexNumber[NumSamples];

                // Specifies the reference location within the acquired record from which to begin fetching. ( at the first sample acquired )
                status = Rfsa.SetFetchRelativeTo("0", niRFSAConstants.FirstSample);
                ErrorCheckRFSA(Rfsa, status);
                                 
                // Fetch IQ data            
                status = Rfsa.ReadIQSingleRecordComplexF64("", timeout, capturedIQ, NumSamples, out captureInfo);
                ErrorCheckRFSA(Rfsa, status);

                // Check to see if the number of samples matches the number of samples returned.
                if (status == 0 && captureInfo.actualSamples == NumSamples)
                {
                    // Fetch of IQ samples was successful. Now copy the complex values into a separate I and Q array.
                    for (int i = 0; i < NumSamples; i++)
                    {
                        iBuffer[i] = capturedIQ[i].Real;
                        qBuffer[i] = capturedIQ[i].Imaginary;
                    }
                }
                else
                {
                    Console.WriteLine("An error occured during capture ( the number of samples returned does not match what was asked for )!");
                    status = Rfsa.Abort();
                    ErrorCheckRFSA(Rfsa, status);
                }

            }
            else
            {
                Console.WriteLine("An error occured during capture!");
                status = Rfsa.Abort();
                ErrorCheckRFSA(Rfsa, status);
            }
        }

        /// <summary>
        /// The CalculateAveragePower method makes a basic power measurement over the specified I and Q samples, applies the correction factor.      
        /// </summary>        
        /// <param name="iData">Array of I samples.</param>
        /// <param name="qData">Array of Q samples</param>
        /// <param name="startIndex">Start index within the IQ arrays to perform the measurement from.</param>
        /// <param name="finishIndex">Finish index within the IQ arrays to perform the measurement to.</param>                  
        /// <returns>Average power in dBm</returns>
        static private double CalculateAveragePower(double[] iData, double[] qData, int startIndex, int finishIndex, double levelCorrection)
        {          
            int totalSamples;

            double powerInWatts = 0.0;            
            double power = 0.0;
            double averagePowerResult = double.NaN;    

            for (int index = startIndex; index < finishIndex; index++)
            {
                try
                {
                    // Calculate the power in Watts for each IQ pair.
                    power = iData[index] * iData[index] + qData[index] * qData[index];
                }
                catch (Exception ex)
                {
                    throw new ApplicationException("Power can not be calcuated with IQ buffer : " + ex.Message);
                }

                powerInWatts += power;                
            }


            // Work out how many samples the power calculation was performed on.
            totalSamples = finishIndex - startIndex;

            // Work out the average power in Watts.
            powerInWatts = powerInWatts / totalSamples;

            // Convert power in Watts to dBm.
            averagePowerResult = 10.0 * Math.Log10(powerInWatts) + levelCorrection;

            return averagePowerResult;
        }
         
        /// <summary>
        /// Console App entry point.
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            niRFSG Rfsg = null;
            niRFSA Rfsa = null;
            double frequency        = 2.0e9; // 2 GHz.
	        double referenceLevel   = 20; // 20 dBm
	        double dwellTime        = 1e-3; // 1 ms.
	        double sampleRate        = 40e6; // 40 MHz
	        double averagePowerdBm;
	        int numberOfSamples = (int)(dwellTime * sampleRate);
            double[] iData = new double[numberOfSamples];
            double[] qData = new double[numberOfSamples];

            try
            {
                // Set the window size a little larger to make things easier to read.
                Console.SetWindowSize(150, 50);
                Console.WriteLine("VST Example 2 - Configure and perform basic capture using RFSA");
                Console.WriteLine("--------------------------------------------------------------");
                Console.WriteLine();
                Console.WriteLine("Booting and configuring RFSA.");
                BootRFSA(out Rfsa);
                
                // Capture some IQ.
                Capture(Rfsa, frequency, referenceLevel, numberOfSamples, sampleRate, iData, qData);

                // Now Calculate the Average Power Over the Capture
	            averagePowerdBm = CalculateAveragePower(iData,qData,0,numberOfSamples-1,0);

                Console.WriteLine("\n\nAverage Power of Capture : {0} dBm\n\n", averagePowerdBm.ToString("##.###"));                
            }
            catch (Exception ex)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(ex.Message);
                Console.ForegroundColor = ConsoleColor.Gray;
            }
            finally
            {
                Console.WriteLine("Closing RFSA.");
                CloseRFSA(Rfsa);
                Console.WriteLine("Closing RFSG.");
                CloseRFSG(Rfsg);
                Console.WriteLine();
                Console.WriteLine();
                Console.WriteLine("Press Any Key To Exit!");
                Console.ReadLine();
            }
        }
    }
}
