﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

using NationalInstruments.ModularInstruments.Interop;

namespace Example_1
{
    class Program
    {
        /// <summary>
        /// Using a RFSA VISA session and a status code the status code is checked to see if a warning has occured.
        /// </summary>
        /// <param name="Rfsa">RFSA instance</param>
        /// <param name="Status">Status code</param>
        static private void ErrorCheckRFSA(niRFSA Rfsa, int Status)
        {
            StringBuilder sb = new StringBuilder(256);

            if (Status > 0) // Warning
            {
                // Retrieve the error message based on the status code.
                Rfsa.ErrorMessage(Status, sb);

                Console.WriteLine("Warning : " + sb.ToString());
            }
        }

        /// <summary>
        ///  Using a RFSG VISA session and a status code the status code is checked to see if a warning has occured.
        /// </summary>
        /// <param name="Rfsg">RFSG instance</param>
        /// <param name="Status">Status code</param>
        static private void ErrorCheckRFSG(niRFSG Rfsg, int Status)
        {
            StringBuilder sb = new StringBuilder(256);

            if (Status > 0) // Warning
            {
                // Retrieve the error message based on the status code.
                Rfsg.ErrorMessage(Status, sb);

                Console.WriteLine("Warning : " + sb.ToString());
            }
        }

        /// <summary>
        /// Boots the RFSA within the VST.
        /// </summary>
        /// <param name="Rfsa">Instance of the Rfsa object.</param>
        static private void BootRFSA(out niRFSA Rfsa)
        {
            int status;
            double referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

            // Initiate VSA session		
            Rfsa = new niRFSA("RIO0", true, false);

            // Configure 10MHz reference
            status = Rfsa.ConfigureRefClock("OnboardClock", referenceClockRate); // "OnboardClock" or "RefIn" or "PXI_CLK" or "None"
            ErrorCheckRFSA(Rfsa, status);

            // Commit the settings to make clock settings active
            status = Rfsa.Commit();
            ErrorCheckRFSA(Rfsa, status);

            Console.WriteLine("VST - RFSA Booted and Configured for on-board 10 MHz reference clock. : ");
        }

        /// <summary>
        /// Boots the RFSG within the VST.
        /// </summary>
        /// <param name="Rfsa">Instance of the Rfsg object.</param>
        static private void BootRFSG(out niRFSG Rfsg)
        {
            int status;
            double referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

            // Initiate VSA session		
            Rfsg = new niRFSG("RIO0", true, false);

            // Configure 10MHz reference
            status = Rfsg.ConfigureRefClock("OnboardClock", referenceClockRate); // "OnboardClock" or "RefIn" or "PXI_CLK" or "None"
            ErrorCheckRFSG(Rfsg, status);

            // Commit the settings to make clock settings active
            status = Rfsg.Commit();
            ErrorCheckRFSG(Rfsg, status);

            Console.WriteLine("VST - RFSG Booted and Configured for on-board 10 MHz reference clock. : ");
        }


        /// <summary>
        /// Closes the RFSA session.
        /// </summary>
        /// <param name="Rfsa">RFSA instance</param>
        static private void CloseRFSA(niRFSA Rfsa)
        {
            if (Rfsa != null)
            {
                // Close session
                Rfsa.Close();
            }
        }

        /// <summary>
        /// Closes the RFSG session.
        /// </summary>
        /// <param name="Rfsg">RFSG instance.</param>
        static private void CloseRFSG(niRFSG Rfsg)
        {
            if (Rfsg != null)
            {
                // Close session
                Rfsg.close();
            }
        }

        /// <summary>
        /// Generates a CW tone at a given frequency and level.
        /// </summary>
        /// <param name="Rfsg">RFSG instance</param>
        /// <param name="Frequency">Frequency to play CW tone at.</param>
        /// <param name="Level">Level in dBm the CW tone will be played at.</param>
        static private void GenerateCwTone(niRFSG Rfsg, double Frequency, double Level)
        {
            int status;
            bool done;

            // Aborts generation & put VSG in configuration mode
            status = Rfsg.Abort();
            ErrorCheckRFSG(Rfsg, status);

            //
            // Configure all settings that require the VSG to be in configuration mode
            //

            // Set generation mode to CW
            status = Rfsg.ConfigureGenerationMode(niRFSGConstants.Cw);
            ErrorCheckRFSG(Rfsg, status);

            // Disable start trigger, so generation will commence immediately after _Initiate() called
            status = Rfsg.DisableStartTrigger();
            ErrorCheckRFSG(Rfsg, status);

            // Configure frequency & level using higher level function
            status = Rfsg.ConfigureRF(Frequency, Level);
            ErrorCheckRFSG(Rfsg, status);

            // Note Level can also be set using a low level attribute.
            status = Rfsg.SetPowerLevel("0", Level);
            ErrorCheckRFSG(Rfsg, status);

            // Note Frequency can also be set using a low level attribute.
            status = Rfsg.SetFrequency("0", Frequency);
            ErrorCheckRFSG(Rfsg, status);

            // Enable output
            status = Rfsg.ConfigureOutputEnabled(true);
            ErrorCheckRFSG(Rfsg, status);

            // Note output state can also be configured using low level attributes.
            status = Rfsg.SetOutputEnabled("0", true);
            ErrorCheckRFSG(Rfsg, status);

            // Initiate generation (VSG now in generation mode)
            status = Rfsg.Initiate();
            ErrorCheckRFSG(Rfsg, status);

            // Check Generation Status. ( returns 0 (generation active) )
            status = Rfsg.CheckGenerationStatus(out done);
            ErrorCheckRFSG(Rfsg, status);
        }
  
        /// <summary>
        /// Reads in an IQ waveform specified as a .csv file and returns to filled in I and Q vectors.
        /// </summary>
        /// <param name="fileName">Filename of file to read in.</param>
        /// <param name="I">Double array to hold I values.</param>
        /// <param name="Q">Double array to hold Q values.</param>
        static private void ReadCsvFile(string fileName, out double[] I, out double[] Q)
        {
            List<double> iData = new List<double>();
            List<double> qData = new List<double>();
            string[] tmp;

            StreamReader reader = new StreamReader(fileName);
            string line;

            while (reader.EndOfStream != true)
            {
                line = reader.ReadLine();
                tmp = line.Split(',');
                iData.Add(double.Parse(tmp[0]));
                qData.Add(double.Parse(tmp[1]));
            }

            I = iData.ToArray();
            Q = qData.ToArray();

        }

        /// <summary>
        /// Plays back the specified .csv file.
        /// </summary>
        /// <param name="Rfsg">RFSG instance</param>
        /// <param name="fileName">Filename and path of csv file.</param>
        /// <param name="frequency">Frequency in Hz</param>
        /// <param name="level">Level in dBm</param>
        /// <param name="sampleRate">Sample Rate in Hz that the waveform it to played back at.</param>
        static private void PlayIQWaveform(niRFSG Rfsg, string fileName, double frequency, double level, double sampleRate)
        {
            int status;
            bool done;
            double[] I;
            double[] Q;

            // Load in am IQ waveform from file that is specified with a IQ pair per line deliminated with a ','
            ReadCsvFile(fileName, out I, out Q);

            // Aborts generation & put VSG in configuration mode
            status = Rfsg.Abort();
            ErrorCheckRFSG(Rfsg, status);

            //
            // Configure all settings that require the VSG to be in configuration mode
            //

            // RMS mode will automatically rescale ARB files to maximise DAC dynamic range.
            // PEAK mode will treat the RF level as being the peak value, so the RMS value will usually be lower.  ARB files are not re-scaled.
            status = Rfsg.ConfigurePowerLevelType(niRFSGConstants.PeakPower);
            ErrorCheckRFSG(Rfsg, status);

            // Clear All wavefroms so we don't get an error if the waveform is already loaded.
            status = Rfsg.ClearAllArbWaveforms();
            ErrorCheckRFSG(Rfsg, status);

            // Write Arb file to RFSG.
            status = Rfsg.WriteArbWaveform("gsmtsc0", I.Length, I, Q, false);
            ErrorCheckRFSG(Rfsg, status);

            // Set ARB playback sample rate
            status = Rfsg.SetIqRate("0", sampleRate);
            ErrorCheckRFSG(Rfsg, status);

            // Set generation mode to ARB Waveform.
            status = Rfsg.ConfigureGenerationMode(niRFSGConstants.ArbWaveform);
            ErrorCheckRFSG(Rfsg, status);

            // Configure the ARB to playback continuously or a finite numberof times. False for continuous playback true to play back n-times.
            status = Rfsg.SetArbWaveformRepeatCountIsFinite("0", false);
            ErrorCheckRFSG(Rfsg, status);

            // Configure number of times ARB file is played
            status = Rfsg.SetArbWaveformRepeatCount("0", 1000);
            ErrorCheckRFSG(Rfsg, status);

            // Disable start trigger, so generation will commence immediately after _Initiate() called
            status = Rfsg.DisableStartTrigger();
            ErrorCheckRFSG(Rfsg, status);

            // Configure frequency & level using higher level function
            status = Rfsg.ConfigureRF(frequency, level);
            ErrorCheckRFSG(Rfsg, status);

            // Enable output
            status = Rfsg.ConfigureOutputEnabled(true);
            ErrorCheckRFSG(Rfsg, status);

            // Enable software start trigger
            status = Rfsg.ConfigureSoftwareStartTrigger();
            ErrorCheckRFSG(Rfsg, status);

            // Initiate generation (VSG now in generation mode)
            status = Rfsg.Initiate();
            ErrorCheckRFSG(Rfsg, status);

            // Check Generation Status. ( returns false (generation active) )
            status = Rfsg.CheckGenerationStatus(out done);
            ErrorCheckRFSG(Rfsg, status);

            // VSG now in generation mode but awaiting a software trigger to commence ARB playback
            status = Rfsg.SendSoftwareEdgeTrigger(niRFSGConstants.StartTrigger, "");// SendSoftwareEdgeTrigger(NIRFSG_VAL_START_TRIGGER, ""); // send software trigger
            ErrorCheckRFSG(Rfsg, status);
        }

        /// <summary>
        /// Console App entry point.
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            niRFSG Rfsg = null;
            double sampleRate = 20e6; // 20 MHz.
            double frequency = 1e9;	// 1 GHz
            double level = -20;	// -20 dBm

            try
            {
                // Set the window size a little larger to make things easier to read.
                Console.SetWindowSize(150, 50);
                Console.WriteLine("VST Example 3 - Generate CW Tone or Playback IQ Waveform");
                Console.WriteLine("--------------------------------------------------------");
                Console.WriteLine();
                Console.WriteLine("Booting and configuring RFSG.");
                BootRFSG(out Rfsg);

                while (true)
                {
                    Console.WriteLine();
                    Console.WriteLine();
                    Console.WriteLine("Please Select");
                    Console.WriteLine("-------------");
                    Console.WriteLine("1. Generate CW Tone");
                    Console.WriteLine("2. Playback IQ Waveform");
                    Console.WriteLine("3. Exit");

                    ConsoleKeyInfo keyPress = Console.ReadKey();

                    if (keyPress.KeyChar == '1')
                    {
                        GenerateCwTone(Rfsg, frequency, level);
                    }
                    else if (keyPress.KeyChar == '2')
                    {
                        PlayIQWaveform(Rfsg, "gsmtsc0.csv", frequency, level, sampleRate);
                    }
                    else if (keyPress.KeyChar == '3')
                    {
                        // Exit loop
                        break;
                    }
                    else
                    {
                        Console.WriteLine(keyPress);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(ex.Message);
                Console.ForegroundColor = ConsoleColor.Gray;
            }
            finally
            {
                Console.WriteLine("Closing RFSG.");
                CloseRFSG(Rfsg);
                Console.WriteLine();
                Console.WriteLine();
                Console.WriteLine("Press Any Key To Exit!");
                Console.ReadLine();
            }
        }
    }
}
